@@ -306,17 +306,15 @@ class Corr:
         else:
             symmetric_corr = self.matrix_symmetric()
 
+        G0 = np.vectorize(lambda x: x.value)(symmetric_corr[t0])
+        np.linalg.cholesky(G0)  # Check if matrix G0 is positive-semidefinite.
+
         if sort is None:
             if (ts is None):
                 raise Exception("ts is required if sort=None.")
             if (self.content[t0] is None) or (self.content[ts] is None):
                 raise Exception("Corr not defined at t0/ts.")
-            G0, Gt = np.empty([self.N, self.N], dtype="double"), np.empty([self.N, self.N], dtype="double")
-            for i in range(self.N):
-                for j in range(self.N):
-                    G0[i, j] = symmetric_corr[t0][i, j].value
-                    Gt[i, j] = symmetric_corr[ts][i, j].value
-
+            Gt = np.vectorize(lambda x: x.value)(symmetric_corr[ts])
             reordered_vecs = _GEVP_solver(Gt, G0)
 
         elif sort in ["Eigenvalue", "Eigenvector"]:
@@ -325,17 +323,12 @@ class Corr:
             all_vecs = [None] * (t0 + 1)
             for t in range(t0 + 1, self.T):
                 try:
-                    G0, Gt = np.empty([self.N, self.N], dtype="double"), np.empty([self.N, self.N], dtype="double")
-                    for i in range(self.N):
-                        for j in range(self.N):
-                            G0[i, j] = symmetric_corr[t0][i, j].value
-                            Gt[i, j] = symmetric_corr[t][i, j].value
-
+                    Gt = np.vectorize(lambda x: x.value)(symmetric_corr[t])
                     all_vecs.append(_GEVP_solver(Gt, G0))
                 except Exception:
                     all_vecs.append(None)
             if sort == "Eigenvector":
-                if (ts is None):
+                if ts is None:
                     raise Exception("ts is required for the Eigenvector sorting method.")
                 all_vecs = _sort_vectors(all_vecs, ts)
 
@@ -523,7 +516,7 @@ class Corr:
         ----------
         variant : str
             decides which definition of the finite differences derivative is used.
-            Available choice: symmetric, forward, backward, improved, default: symmetric
+            Available choice: symmetric, forward, backward, improved, log, default: symmetric
         """
         if self.N != 1:
             raise Exception("deriv only implemented for one-dimensional correlators.")
@@ -567,6 +560,17 @@ class Corr:
             if (all([x is None for x in newcontent])):
                 raise Exception('Derivative is undefined at all timeslices')
             return Corr(newcontent, padding=[2, 2])
+        elif variant == 'log':
+            newcontent = []
+            for t in range(self.T):
+                if (self.content[t] is None) or (self.content[t] <= 0):
+                    newcontent.append(None)
+                else:
+                    newcontent.append(np.log(self.content[t]))
+            if (all([x is None for x in newcontent])):
+                raise Exception("Log is undefined at all timeslices")
+            logcorr = Corr(newcontent)
+            return self * logcorr.deriv('symmetric')
         else:
             raise Exception("Unknown variant.")
 
@@ -577,7 +581,7 @@ class Corr:
         ----------
         variant : str
             decides which definition of the finite differences derivative is used.
-            Available choice: symmetric, improved, default: symmetric
+            Available choice: symmetric, improved, log, default: symmetric
         """
         if self.N != 1:
             raise Exception("second_deriv only implemented for one-dimensional correlators.")
@@ -601,6 +605,17 @@ class Corr:
             if (all([x is None for x in newcontent])):
                 raise Exception("Derivative is undefined at all timeslices")
             return Corr(newcontent, padding=[2, 2])
+        elif variant == 'log':
+            newcontent = []
+            for t in range(self.T):
+                if (self.content[t] is None) or (self.content[t] <= 0):
+                    newcontent.append(None)
+                else:
+                    newcontent.append(np.log(self.content[t]))
+            if (all([x is None for x in newcontent])):
+                raise Exception("Log is undefined at all timeslices")
+            logcorr = Corr(newcontent)
+            return self * (logcorr.second_deriv('symmetric') + (logcorr.deriv('symmetric'))**2)
         else:
             raise Exception("Unknown variant.")
 
@@ -615,6 +630,7 @@ class Corr:
             sinh : Use anti-periodicitiy of the correlator by solving C(t) / C(t+1) = sinh(m * (t - T/2)) / sinh(m * (t + 1 - T/2)) for m.
             See, e.g., arXiv:1205.5380
             arccosh : Uses the explicit form of the symmetrized correlator (not recommended)
+            logsym: uses the symmetric effective mass log(C(t-1) / C(t+1))/2
         guess : float
             guess for the root finder, only relevant for the root variant
         """
@@ -634,6 +650,20 @@ class Corr:
 
             return np.log(Corr(newcontent, padding=[0, 1]))
 
+        elif variant == 'logsym':
+            newcontent = []
+            for t in range(1, self.T - 1):
+                if ((self.content[t - 1] is None) or (self.content[t + 1] is None)) or (self.content[t + 1][0].value == 0):
+                    newcontent.append(None)
+                elif self.content[t - 1][0].value / self.content[t + 1][0].value < 0:
+                    newcontent.append(None)
+                else:
+                    newcontent.append(self.content[t - 1] / self.content[t + 1])
+            if (all([x is None for x in newcontent])):
+                raise Exception('m_eff is undefined at all timeslices')
+
+            return np.log(Corr(newcontent, padding=[1, 1])) / 2
+
         elif variant in ['periodic', 'cosh', 'sinh']:
             if variant in ['periodic', 'cosh']:
                 func = anp.cosh
