@@ -534,7 +534,7 @@ def test_merge_intersection():
         assert pe.obs._merge_idx(idl_list) == pe.obs._intersection_idx(idl_list)
 
 
-def test_intersection_collapse():
+def test_intersection_reduce():
     range1 = range(1, 2000, 2)
     range2 = range(2, 2001, 8)
 
@@ -542,7 +542,7 @@ def test_intersection_collapse():
     obs_merge = obs1 + pe.Obs([np.random.normal(1.0, 0.1, len(range2))], ["ens"], idl=[range2])
 
     intersection = pe.obs._intersection_idx([o.idl["ens"] for o in [obs1, obs_merge]])
-    coll = pe.obs._collapse_deltas_for_merge(obs_merge.deltas["ens"], obs_merge.idl["ens"], len(obs_merge.idl["ens"]), range1)
+    coll = pe.obs._reduce_deltas(obs_merge.deltas["ens"], obs_merge.idl["ens"], range1)
 
     assert np.all(coll == obs1.deltas["ens"])
 
@@ -998,6 +998,12 @@ def test_cobs_array():
     cobs / np.ones((4, 4))
 
 
+def test_details_tau_no_error():
+    tt = pe.Obs([np.random.rand(500)], ["ens"])
+    tt.gamma_method(S=0)
+    tt.details()
+
+
 def test_hash():
     obs = pe.pseudo_Obs(0.3, 0.1, "test") + pe.Obs([np.random.normal(2.3, 0.2, 200)], ["test2"], [range(1, 400, 2)])
     o1 = obs + pe.cov_Obs(0.0, 0.1, "co") + pe.cov_Obs(0.0, 0.8, "co2")
